' COM Port Demo

' Software License Agreement
'
' Licensor grants any person obtaining a copy of this software ("You") 
' a worldwide, royalty-free, non-exclusive license, for the duration of 
' the copyright, free of charge, to store and execute the Software in a 
' computer system and to incorporate the Software or any portion of it 
' in computer programs You write.   

' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
' IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
' AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
' OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
' THE SOFTWARE.

' Author               Date        Comment
' Jan Axelson           1/2/08     Original

' This application demonstrates how to send text commands via a COM port
' and receive and display responses.

' A form displays combo boxes for selecting a COM port and bit rate.

' This application and PIC microcontroller code to communicate with it 
' are available from www.Lvr.com

Option Explicit On
Option Strict On
Imports System.IO.Ports

Public Class Form1

    Dim myComPort As New SerialPort

    ''' <summary>
    ''' Call a routine to write a command to turn off an LED and read the response.
    ''' </summary>
    ''' 
    Private Sub btnOff_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOff.Click

        SendCommand("L10")

    End Sub

    ''' <summary>
    ''' Call a routint write a command to turn on an LED and read the response.
    ''' </summary>
    ''' 
    Private Sub btnOn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOn.Click

        SendCommand("L11")

    End Sub

    ''' <summary>
    ''' If myComPort is open, finish transmitting.
    ''' Exiting the Using block closes the port and releases its resources.
    ''' </summary>
    ''' 
    Sub CloseComPort()

        Try

            Using myComPort

                If (Not (myComPort Is Nothing)) Then

                    ' The COM port exists.

                    If myComPort.IsOpen Then

                        ' Wait for the transmit buffer to empty.

                        Do While (myComPort.BytesToWrite > 0)
                        Loop

                    End If
                End If

            End Using

        Catch ex As UnauthorizedAccessException

            ' The port may have been removed. Ignore.

        End Try

    End Sub

    ''' <summary>
    ''' Set the BaudRate property of myComPort to match the bit rate selected in the combo box.
    ''' </summary>
    ''' 
    Private Sub cmbBitRate_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbBitRate.SelectedIndexChanged

        myComPort.BaudRate = CInt(cmbBitRate.SelectedItem)

    End Sub

    ''' <summary>
    ''' If the previously selected COM port is open, close it.
    ''' Set the PortName property of myComPort to match the port selected in the combo box.
    ''' Call a routine to open the port.
    ''' </summary>
    ''' 
    Private Sub cmbPorts_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmbPorts.SelectedIndexChanged

        CloseComPort()
        myComPort.PortName = cmbPorts.SelectedItem.ToString
        OpenComPort()
    End Sub

    ''' <summary>
    ''' Call a routine to close the COM port.
    ''' </summary>

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        CloseComPort()

    End Sub

    ''' <summary>
    ''' Call routines to initalize the form and open the selected COM port.
    ''' </summary>
    ''' 
    Private Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        InitializeForm()
        OpenComPort()

    End Sub

    ''' <summary>
    ''' Set up the form and select a default port and bit rate.
    ''' </summary>

    Sub InitializeForm()

        Dim bitRates(9) As Integer
        Dim nameArray() As String

        ' Find the COM ports on the system.

        nameArray = SerialPort.GetPortNames
        Array.Sort(nameArray)

        ' Fill a combo box with the port names.

        cmbPorts.DataSource = nameArray
        cmbPorts.DropDownStyle = ComboBoxStyle.DropDownList

        ' Select a default port.

        cmbPorts.SelectedIndex = 1

        'Bit rates to select from.

        bitRates(0) = 300
        bitRates(1) = 600
        bitRates(2) = 1200
        bitRates(3) = 2400
        bitRates(4) = 9600
        bitRates(5) = 14400
        bitRates(6) = 19200
        bitRates(7) = 38400
        bitRates(8) = 57600
        bitRates(9) = 115200

        'Place the bit rates in a combo box.

        cmbBitRate.DataSource = bitRates
        cmbBitRate.DropDownStyle = ComboBoxStyle.DropDownList

        ' Select a default bit rate.

        cmbBitRate.SelectedItem = 1200

    End Sub

    ''' <summary>
    ''' Set port parameters and open the COM port 
    ''' associated with the SerialPort object myComPort.
    ''' </summary>

    Sub OpenComPort()

        Try
            ' Get the selected COM port's name from the combo box.

            If Not myComPort.IsOpen Then
                myComPort.PortName = cmbPorts.SelectedItem.ToString

                ' Get the selected bit rate from the combo box.

                If cmbBitRate.SelectedIndex > 0 Then
                    myComPort.BaudRate = CInt(cmbBitRate.SelectedItem)
                End If

                ' Set other port parameters.

                myComPort.Parity = Parity.None
                myComPort.DataBits = 8
                myComPort.StopBits = StopBits.One
                myComPort.Handshake = Handshake.None

                myComPort.ReadTimeout = 3000
                myComPort.WriteTimeout = 5000

                ' Open the port.

                myComPort.Open()

            End If

        Catch ex As InvalidOperationException
            MessageBox.Show(ex.Message)

        Catch ex As UnauthorizedAccessException
            MessageBox.Show(ex.Message)

        Catch ex As System.IO.IOException
            MessageBox.Show(ex.Message)

        End Try
    End Sub

    ''' <summary>
    ''' Write a command to the SerialPort object and read the response.
    ''' </summary>
    ''' <param name= "command"> The command to send. </param>

    Private Sub SendCommand(ByVal command As String)

        Dim response As String

        Try

            myComPort.WriteLine(command)
            response = myComPort.ReadLine

            Select Case response

                Case "0"

                    lblStatus.Text = "LED 1 is OFF"

                Case "1"

                    lblStatus.Text = "LED 1 is ON"

                Case Else

            End Select

        Catch ex As TimeoutException
            MessageBox.Show(ex.Message)

        Catch ex As InvalidOperationException
            MessageBox.Show(ex.Message)

        Catch ex As UnauthorizedAccessException
            MessageBox.Show(ex.Message)

        End Try
    End Sub

End Class
